#include "render_area.hpp"
#include "bezier/bezier.hpp"
#include "bezier/cercle.hpp"

#include <GL/glu.h>
#include <iostream>

#include <QPainter>
#include <QMouseEvent>


render_area::render_area(QWidget *parent)
    :QWidget(parent)
{
    //fond blanc
    setBackgroundRole(QPalette::Base);
    setAutoFillBackground(true);


    vertex_selected=-1;
    is_closest=-1;
    is_init=false;
    is_clicked=false;

}

void render_area::init_fig()
{

    std::cout<<"Init geometric figures ..."<<std::endl;

    cercle* cercle_0=new cercle(vec2(1.9,0.8),0.2);
    cercle* cercle_1=new cercle(vec2(-0.2,-0.4),0.2);
    cercle* cercle_2=new cercle(vec2(-1.2,0.7),0.3);

    objects.push_back(cercle_0);
    objects.push_back(cercle_1);
    objects.push_back(cercle_2);

    bezier<vec2>* bezier_0=new bezier<vec2>(vec2(0,0),vec2(0.2,0.5),vec2(-0.2,0.8),vec2(-0.5,0.5));
    bezier<vec2>* bezier_1=new bezier<vec2>(vec2(-2.3,0),vec2(-2.1,-0.5),vec2(-1.8,-0.5),vec2(-1.6,0.3));
    bezier<vec2>* bezier_2=new bezier<vec2>(vec2(1.3,0),vec2(2.0,-0.5),vec2(1.0,-0.8),vec2(2.0,0.2));
    objects.push_back(bezier_0);
    objects.push_back(bezier_1);
    objects.push_back(bezier_2);

    point_sets.resize(objects.size());
    int N_sample=50;
    for(int k_obj=0,N_obj=objects.size();k_obj<N_obj;++k_obj)
    {
        point_sets[k_obj].resize(N_sample);
        const objet_geometrique& current_object=*objects[k_obj];
        for(int k=0;k<N_sample;++k)
        {
            float s=static_cast<float>(k)/(N_sample-1);
            const vec2 p=coord_to_axis(current_object(s));
            point_sets[k_obj][k]=p;
        }
    }


    std::cout<<"Init OK"<<std::endl;

    is_init=true;
}

render_area::~render_area()
{}



void render_area::paintEvent(QPaintEvent*)
{

    if(is_init==false)
        init_fig();

    //la classe qui permet de "dessiner"
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //le pinceau de dessin
    QPen pen;



    pen.setWidth(1.0);
    pen.setColor(Qt::black);
    painter.setPen(pen);

    for(int k_obj=0,N_obj=objects.size();k_obj<N_obj;++k_obj)
    {
        const std::vector<vec2>& current_curve=point_sets[k_obj];
        for(int k=0,N_sample=current_curve.size();k<N_sample-1;++k)
        {
            const vec2& p0=current_curve[k];
            const vec2& p1=current_curve[k+1];
            painter.drawLine(p0.x,p0.y,p1.x,p1.y);
        }
    }

    pen.setWidth(1.0);
    pen.setColor(Qt::red);
    painter.setPen(pen);
    if(is_clicked==true)
    {
        painter.drawLine(mouse_point.x,mouse_point.y,
                         closest_point.x,closest_point.y);
    }

}

//souris appuyee qui se deplace
void render_area::mouseMoveEvent(QMouseEvent *event)
{
    int x=event->x();
    int y=event->y();

    mouse_point=vec2(x,y);
    const vec2 p_click=axis_to_coord(mouse_point);

    vec2 closest=vec2();
    float dist_closest=0.0f;
    for(int k_obj=0,N_obj=objects.size();k_obj<N_obj;++k_obj)
    {
        const objet_geometrique& current_object=*objects[k_obj];

        closest=current_object.closest_point(p_click);
        float d=norm(closest-p_click);
        if(k_obj==0 || d<dist_closest)
        {
            dist_closest=d;
            closest_point=closest;
        }

    }

    closest_point=coord_to_axis(closest_point);




    repaint();
}

void render_area::mouseReleaseEvent(QMouseEvent *)
{
    is_clicked=false;
    repaint();
}

//simple click souris
void render_area::mousePressEvent(QMouseEvent *)
{
    is_clicked=true;

    repaint();
}

vec2 render_area::coord_to_axis(const vec2& p) const
{
    const float w=this->width();
    const float h=this->height();
    const float L=150.0f;
    vec2 p_axis=p;
    p_axis.y*=-1.0f;
    p_axis*=L;
    p_axis.y+=h/2.0f;
    p_axis.x+=w/2.0f;
    return p_axis;
}

vec2 render_area::axis_to_coord(const vec2& p) const
{
    const float w=this->width();
    const float h=this->height();
    const float L=150.0f;
    vec2 p_coord=p;
    p_coord.x-=w/2.0f;
    p_coord.y-=h/2.0f;
    p_coord/=L;
    p_coord.y*=-1.0f;
    return p_coord;
}
